/**
 * Listens for page turn events as well as
 * maintaining server info structures for each tab.
 */
function SwitcherListener() {
    this.tabContexts = new Array(); //this array stores the information for each tab.
    this.devserverarray = new Array(); //this stores all development-servers
    this.liveserverarray = new Array(); //this stores all live-servers
}

/**
 * Called when the browser is actually up and loaded
 */
SwitcherListener.prototype.Initialize = function(aEvent) {
    // Only load if this is the chrome load event
    if (aEvent.originalTarget != document) {
        return;
    }

    ServerSwitcher.InitializeServers();

    // Make sure we hear about page turn events
    var contentArea = document.getElementById("content");
    contentArea.addProgressListener(ServerSwitcher, Components.interfaces.nsIWebProgress.NOTIFY_ALL);

    window.addEventListener("keypress", handleEventWrapper, true);
}

/**
 * Load servers in array
 */
SwitcherListener.prototype.InitializeServers = function(aEvent) {

    var devserver  = null;
    var liveserver = null;
    var devserverValue  = null;
    var liveserverValue = null;
    //var serverregex = /^(https?:\/\/|file:\/\/\/)?[\w.-]+(:[0-9]*)?(\/[\w.-~]+)*$/; // change also in serveradd.js!
    var serverregex = /^(https?:\/\/|file:\/\/\/)?[^:/]+(:[0-9]*)?(\/[^/]+)*$/; // change also in serveradd.js!

    ServerSwitcher.devserverarray = new Array();
    ServerSwitcher.liveserverarray = new Array();

    var serverCount      = 0;

    if(webdeveloper_isPreferenceSet("serverswitcher.servers.count"))
    {
        serverCount = webdeveloper_getIntegerPreference("serverswitcher.servers.count", true);
    }

    // save all live & dev-servers
    for(var i = 1; i <= serverCount; i++)
    {
        devserver    = "serverswitcher.servers." + i + ".devserver";
        liveserver   = "serverswitcher.servers." + i + ".liveserver";

        // If the devserver is set
        if(webdeveloper_isPreferenceSet(devserver))
        {
            var serverstring = webdeveloper_getStringPreference(devserver, true);
            var matches = serverstring.match(serverregex);
            if(typeof(matches[1]) == 'undefined') serverstring = 'http://'+serverstring;
            devserverValue = serverstring;
        }
        else
        {
            devserverValue = null;
        }

        // If the liveserver is set
        if(webdeveloper_isPreferenceSet(liveserver))
        {
            var serverstring = webdeveloper_getStringPreference(liveserver, true);
            var matches = serverstring.match(serverregex);
            if(typeof(matches[1]) == 'undefined') serverstring = 'http://'+serverstring;
            liveserverValue = serverstring;
        }
        else
        {
            liveserverValue = null;
        }

        // If the liveserver and devserver are set
        if(devserverValue && liveserverValue)
        {
            ServerSwitcher.devserverarray.push(devserverValue);
            ServerSwitcher.liveserverarray.push(liveserverValue);
        }
    }

    webdeveloper_setBooleanPreference("serverswitcher.servers.changed", false); // current servers are saved
}


/**
 * Called when the browser is closing down
 */
SwitcherListener.prototype.Destroy = function(aEvent) {
  if (aEvent.originalTarget != document) {
    return;
  }

  // Unhook this listener from browser
  var contentArea = document.getElementById("content");
  contentArea.removeProgressListener(ServerSwitcher);
}


/**
 * This fires when the location bar changes (either a new page or a new tab)
 */
SwitcherListener.prototype.onLocationChange = function(aProgress, aRequest, aURI) {
  // Make sure icon is hidden
  ServerSwitcher.DisableDisplay();

  // Don't do anything If we're not on a valid page (empty tab, about:blank, etc)
  if (aURI.scheme != "http" && aURI.scheme != "https" && aURI.scheme != "file") {
    return;
  }

  // Walk through our array of contexts, see if this is a page turn for an
  // existing tab
  var contentArea = document.getElementById("content");
  var currentBrowser = contentArea.selectedBrowser;
  var foundContext = -1;
  for (var i = 0; i < this.tabContexts.length; i++) {
    if (this.tabContexts[i].browser == currentBrowser) {
      foundContext = i;
      break;
    }
  }

  // If this is a new tab add it now
  if (foundContext == -1) {

    // While we're here, clean up any tabContexts that no longer have a matching
    // tab in the browser
    for (var i = 0; i < this.tabContexts.length; i++) {
      var tabExists = false;
      for (j = 0; j < contentArea.browsers.length; j++) {
        if (contentArea.browsers[j] == this.tabContexts[i].browser) {
          tabExists = true;
          break;
        }
      }

      // Couldn't find a browser tab for this context, remove this context
      if (!tabExists) {
        var removedContext = this.tabContexts.splice(i, 1);
        removedContext[0].Destroy();

        // We need to check this index again
        i--;
      }
    }

    // Append a new tab context
    var newContext = new ServerSwitcherTabData(currentBrowser);
    this.tabContexts[this.tabContexts.length] = newContext;
    foundContext = this.tabContexts.length - 1;
  }

  // Select the info for the current context and get it going
  var currentServerSwitcherTabData = this.tabContexts[foundContext];
  currentServerSwitcherTabData.LocationChange(aURI);

  return;
}

/**
 * Disables display elements
 */
SwitcherListener.prototype.DisableDisplay = function() {
  var statusbarIcon = document.getElementById("switcher_icon");
  if (statusbarIcon) {
    var iconContainer = statusbarIcon.parentNode;
    iconContainer.removeChild(statusbarIcon);
  }
}

/**
 * Enables icon
 */
SwitcherListener.prototype.EnableDisplay = function(displayNode, toolTipText) {
    // Kill it if it's already enabled
    ServerSwitcher.DisableDisplay();

    // Add the status bar icon
    var URLBarHbox = document.getElementById("urlbar").childNodes[1];
    URLBarHbox.insertBefore(displayNode, URLBarHbox.firstChild);
    displayNode.addEventListener("click", handleEventWrapper, false);

    // Set tooltips
    if (toolTipText) {
        document.getElementById("switcher_icon").setAttribute("tooltiptext", toolTipText);
    }
}

/**
 * Called when the user clicks on the icon or presses key combo
 */
SwitcherListener.prototype.handleEvent = function(event) {
  var newtab = false; // open in new tab?
  if(event.button == 1 || (event.button == 0 && event.ctrlKey)) { // middle button or CTRL+leftclick
    newtab = true;
  }
  else if(event.button == 0 || (event.ctrlKey && event.shiftKey && event.charCode == '88')); // left mouse or key combo
  else return; // other button? no action.

  var contentArea = document.getElementById("content");
  var currentBrowser = contentArea.selectedBrowser;

  var foundContext = -1;
  for (var i = 0; i < this.tabContexts.length; i++) {
    if (this.tabContexts[i].browser == currentBrowser) {
      foundContext = i;
      break;
    }
  }

  if(foundContext == -1 || (this.tabContexts[i].devserver==false && this.tabContexts[i].liveserver==false)) return; // Context not found or not switchable server

  var newurl = this.tabContexts[i].uri.spec;
  var replacer = new RegExp('^'+this.tabContexts[i].replacepart.replace(/\./g, '\\.'));
  newurl = newurl.replace(replacer, this.tabContexts[i].switchurl);

  // Redirect the browser
  if(newtab) {
    var browser = getBrowser();
    var newtab = browser.addTab(newurl);
    browser.selectedTab = newtab; // focus the new tab
  }
  else getBrowser().loadURI(newurl);
}

SwitcherListener.prototype.onStateChange = function(aProgress, aRequest, aFlag, aStatus) { return 0; },
SwitcherListener.prototype.onProgressChange = function() { return 0; },
SwitcherListener.prototype.onStatusChange = function() { return 0; },
SwitcherListener.prototype.onSecurityChange = function() { return 0; },
SwitcherListener.prototype.onLinkIconAvailable = function() { return 0; },

SwitcherListener.prototype.QueryInterface = function(aIID) {
 if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
     aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
     aIID.equals(Components.interfaces.nsISupports))
   return this;
 throw Components.results.NS_NOINTERFACE;
}





/**
 * ServerSwitcher manages one ServerSwitcherTabData object for each browser tab.
 */
function ServerSwitcherTabData(aBrowser) {
  this.uri = null;  // The last uri
  this.replacepart = ""; // part that should be replaced
  this.switchurl = ""; // server to switch to
  this.devserver = false; // this is a dev-server
  this.liveserver = false; // this is a live-server
  this.browser = aBrowser;
  this.mainElement = null;
}


/**
 * Clean up TabData
 */
ServerSwitcherTabData.prototype.Destroy = function() {
  this.uri = null;
  this.replacepart = "";
  this.switchurl = "";
  this.devserver = false;
  this.liveserver = false;
  this.browser = null;
  this.mainElement = null;
}


/**
 * Given a new URL, check if it is dev- or live-server
 */
ServerSwitcherTabData.prototype.LocationChange = function(aURI) {
    var changed = webdeveloper_getBooleanPreference("serverswitcher.servers.changed", true);
    // Don't do anything if this site is the same as the previous
    if (!changed && this.uri != null && aURI.spec == this.uri.spec) {

        // If the current site has an icon, make it visible
        if (this.mainElement) {
            ServerSwitcher.EnableDisplay(this.mainElement, null);
        } else {
            ServerSwitcher.DisableDisplay();
        }
        // save the current uri
        this.uri = aURI;
        return;
    }

    if(changed) { // check if there are unsaved changes
      ServerSwitcher.InitializeServers();
    }

    this.uri = aURI;
    this.devserver = false;
    this.liveserver = false;
    this.replacepart = "";
    this.switchurl = "";
    this.mainElement = null;

    // Check if this is a dev- or live-server
    for(i=0; i < ServerSwitcher.devserverarray.length; i++) {
        var devregex = new RegExp("^"+ServerSwitcher.devserverarray[i].replace(/\./g, '\\.')+'/'); // points must be escaped in regex, g tells him to search for all occurrences of points not just the first
        var liveregex = new RegExp("^"+ServerSwitcher.liveserverarray[i].replace(/\./g, '\\.')+'/');

        if(aURI.spec.match(devregex)) {
            this.devserver = true;
            this.replacepart = ServerSwitcher.devserverarray[i];
            this.switchurl = ServerSwitcher.liveserverarray[i];
            break;
        }
        else if(aURI.spec.match(liveregex)) {
            this.liveserver = true;
            this.replacepart = ServerSwitcher.liveserverarray[i];
            this.switchurl = ServerSwitcher.devserverarray[i];
            break;
        }
    }


    if(this.devserver || this.liveserver) {
        // Build a new server button

        var switcherURLBarButton = document.createElement("image");

        if(this.devserver) switcherURLBarButton.setAttribute("class", "devserver");
        else switcherURLBarButton.setAttribute("class", "liveserver");

        //switcherURLBarButton.setAttribute("onclick", "ServerSwitcher.handleEvent()");
        switcherURLBarButton.setAttribute("id", "switcher_icon");
        //switcherURLBarButton.addEventListener("click", handleEventWrapper, false);

        // Save button for future tab changes
        this.mainElement = switcherURLBarButton;

        ServerSwitcher.EnableDisplay(this.mainElement, "Switch to "+this.switchurl);
    }
    else {
        ServerSwitcher.DisableDisplay();
    }
}

function handleEventWrapper(event) { // without this, there are some strange bugs
  ServerSwitcher.handleEvent(event);
}
